/**
@license https://github.com/t2ym/live-localizer/blob/master/LICENSE.md
Copyright (c) 2016, Tetsuya Mori <t2y3141592@gmail.com>. All rights reserved.
*/
/**

## live-localizer-browser-storage

`<live-localizer-browser-storage>` element shows the storage icon and its controls for the browser storage.
The element handles save and load operations of XLIFF on the browser storage backed by IndexedDB.

@group I18nBehavior
@element live-localizer-browser-storage
@demo demo/index.html
*/
/*
  FIXME(polymer-modulizer): the above comments were extracted
  from HTML and may be out of place here. Review them and
  then delete this comment!
*/
import '@polymer/polymer/polymer-legacy.js';

import '@polymer/iron-flex-layout/iron-flex-layout-classes.js';
import '@polymer/iron-icon/iron-icon.js';
import '@polymer/iron-icons/iron-icons.js';
import '@polymer/iron-icons/device-icons.js';
import '@polymer/iron-image/iron-image.js';
import '@polymer/paper-icon-button/paper-icon-button.js';
import '@polymer/paper-card/paper-card.js';
import '@polymer/paper-ripple/paper-ripple.js';
import '@polymer/paper-tooltip/paper-tooltip.js';
import '@polymer/paper-fab/paper-fab.js';
import '@polymer/paper-checkbox/paper-checkbox.js';
import './live-localizer-storage-icon.js';
import './draggable-behavior.js';
import Dexie from 'dexie/dist/dexie.es6.js';
import { Polymer as Polymer$0 } from '@polymer/polymer/lib/legacy/polymer-fn.js';
import { html } from '@polymer/polymer/lib/utils/html-tag.js';
import deepcopy from 'deepcopy/dist/deepcopy.js';

Polymer$0({
  importMeta: import.meta,

  _template: html`
    <style include="iron-flex"></style>
    <style include="drag-handle-mode"></style>
    <style include="drag-field"></style>
    <style>
      :host {
        display: inline-block;
        margin-left: 12px;
        
        --live-localizer-default-checkbox-color: dimgrey;
        --paper-checkbox-unchecked-color: var(--live-localizer-color,--live-localizer-default-checkbox-color);
        --paper-checkbox-checked-color: var(--live-localizer-color,--live-localizer-default-checkbox-color);
        --paper-checkbox-label-color:var(--live-localizer-color,--live-localizer-default-checkbox-color);
        --paper-checkbox-label-checked-color: var(--live-localizer-color,--live-localizer-default-checkbox-color);
      }
      .control-panel {
        padding: 12px;
      }
      .control-panel-item {
        margin-bottom: 4px;
      }
    </style>
    <div id="storage-panel" class="layout horizontal">
      <live-localizer-storage-icon id="browser-storage-icon" selected="{{selected}}" icon="{{icon}}" label="{{label}}" badge-label="{{badgeLabel}}" badge-color="{{badgeColor}}" badge-tooltip="Discarded Units" on-tap="onTap" drag-handle-mode="drag" drag-drop-groups="save-targets" drop-targets="load-targets" tooltips="[ &quot;Drag to Load&quot;, &quot;Drop to Save&quot; ]" model="{{model}}"></live-localizer-storage-icon>
      <div class="control-panel layout vertical">
        <div class="control-panel-item">Automatic</div>
        <paper-checkbox class="control-panel-item" checked="{{autoSave}}">Save</paper-checkbox>
        <paper-checkbox class="control-panel-item" checked="{{autoLoad}}">Load</paper-checkbox>
        <div class="flex"></div>
      </div>
      <paper-tooltip id="tooltip" for="storage-panel" offset="-50" manual-mode="">{{tooltip}}</paper-tooltip>
    </div>
`,

  is: 'live-localizer-browser-storage',

  properties: {
    /**
     * name of icon for `<live-localizer-storage-icon>`
     */
    icon: {
      type: String,
      value: 'device:storage'
    },

    /**
     * label of icon for `<live-localizer-storage-icon>`
     */
    label: {
      type: String,
      value: 'Browser Storage'
    },

    /**
     * true if selected
     *
     * Note:
     * - If a file is stored, it is selected.
     */
    selected: {
      type: Boolean,
      value: false,
      reflectToAttribute: true
    },

    /**
     * the model object bound to the `<live-localizer-model>` element
     */
    model: {
      type: Object,
      notify: true
    },

    /**
     * the file object sync'ed with IndexedDB
     */
    file: {
      type: Object,
      notify: true
    },

    /**
     * true if auto saving on loading XLIFF to the running app is enabled
     */
    autoSave: {
      type: Boolean,
      value: true,
      notify: true
    },

    /**
     * true if auto loading on the page loading is enabled
     */
    autoLoad: {
      type: Boolean,
      value: true,
      notify: true
    },

    /**
     * store status of auto loading to avoid multiple auto-loading
     */
    autoLoadStatus: {
      type: Object,
      value: function () { return {}; }
    },

    /**
     * tooltip for the storage
     */
    tooltip: {
      type: String
    }
  },

  observers: [
    'modelReady(model)',
    'updateStats(file)',
    'updateLabel(file)',
    'checkAutoLoad(file,autoLoad)',
    'updateAutoSaveSetting(autoSave)',
    'updateAutoLoadSetting(autoLoad)'
  ],

  /**
   * ready callback
   */
  ready: function () {
    this.$['browser-storage-icon'].storage = this;
    this.onHtmlLangMutationBindThis = this.onHtmlLangMutation.bind(this);
    this.onBundleSetFetchedBindThis = this.onBundleSetFetched.bind(this);
    this.onLoadXliffBindThis = this.onLoadXliff.bind(this);
  },

  /**
   * attached callback
   */
  attached: function () {
    this.isAttached = true;
  },

  /**
   * observer of `this.model`
   *
   * Initialize event handlers and IndexedDB
   */
  modelReady: function (model) {
    if (model && !this.isModelReady) {
      this.model.addEventListener('html-lang-mutation', this.onHtmlLangMutationBindThis);
      this.model.addEventListener('bundle-set-fetched', this.onBundleSetFetchedBindThis);
      this.model.addEventListener('load-xliff', this.onLoadXliffBindThis);
      this.storage = new Dexie('LiveLocalizer' + this.model.startUrl);
      this.storage.version(1).stores({
        files: '&locale',
        settings: '&name'
      });
      this.initializeSettings();
      this.isModelReady = true;
    }
  },

  /**
   * detached callback
   */
  detached: function () {
    if (this.model) {
      this.model.removeEventListener('html-lang-mutation', this.onHtmlLangMutationBindThis);
      this.model.removeEventListener('bundle-set-fetched', this.onBundleSetFetchedBindThis);
      this.model.removeEventListener('load-xliff', this.onLoadXliffBindThis);
    }
  },

  /**
   * observer of `autoSave`
   *
   * Update the `autoSave` setting in IndexedDB
   */
  updateAutoSaveSetting: function (autoSave) {
    if (this.isAttached && this.isAutoSaveInitialized) {
      var it = this;
      this.storage.settings.put({ name: 'autoSave', value: autoSave })
        .then(function () {
          //console.log('store autoSave = ' + autoSave);
          it.fire('browser-storage-autosave-flushed', { value: autoSave });
        })
        .catch(function (error) {
          //console.log('error in autoSave = ' + autoSave, error);
          it.fire('browser-storage-autosave-flushed', { value: autoSave, error: error });
        });
    }
  },

  /**
   * observer of `autoLoad`
   *
   * Update the `autoLoad` setting in IndexedDB
   */
  updateAutoLoadSetting: function (autoLoad) {
    if (this.isAttached && this.isAutoLoadInitialized) {
      var it = this;
      this.storage.settings.put({ name: 'autoLoad', value: autoLoad })
        .then(function () {
          //console.log('store autoLoad = ' + autoLoad);
          it.fire('browser-storage-autoload-flushed', { value: autoLoad });
        })
        .catch(function (error) {
          //console.log('error in autoLoad = ' + autoLoad, error);
          it.fire('browser-storage-autoload-flushed', { value: autoLoad, error: error });
        });
    }
  },

  /**
   * initialize `autoSave` and `autoLoad` settings in IndexedDB
   *
   * @return {Promise}
   */
  initializeSettings: function () {
    var it = this;
    return new Promise(function (resolve, reject) {
      it.storage.settings.toArray()
        .then(function (settings) {
          var autoSaveSetting;
          var autoLoadSetting;
          settings.forEach(function (setting) {
            if (setting.name === 'autoSave') {
              autoSaveSetting = setting;
            }
            else if (setting.name === 'autoLoad') {
              autoLoadSetting = setting;
            }
          });
          if (autoSaveSetting && autoLoadSetting) {
            it.autoSave = autoSaveSetting.value;
            it.autoLoad = autoLoadSetting.value;
            it.isAutoSaveInitialized = true;
            it.isAutoLoadInitialized = true;
            resolve();
          }
          else {
            it.storage.settings.bulkPut([
                { name: 'autoSave', value: it.autoSave },
                { name: 'autoLoad', value: it.autoLoad }
              ])
              .then(function () {
                //console.log('store autoSave = ' + it.autoSave);
                it.isAutoSaveInitialized = true;
                it.isAutoLoadInitialized = true;
                resolve();
              })
              .catch(function (error) {
                //console.log('error in autoSave = ' + it.autoSave, error);
                it.isAutoSaveInitialized = true;
                it.isAutoLoadInitialized = true;
                resolve();
              });
          }
        });
    });
  },

  /**
   * `html-lang-mutation` event handler
   *
   * Load the file for the updated locale into `this.file`
   */
  onHtmlLangMutation: function (e) {
    var file = this.model.selectedFile();
    var noload = true;
    if (file) {
      if (file.locale && file.locale !== this.model.defaultLang) {
        this.loadFile(file, noload)
          .then(function () {
            //console.log('browser-storage: onHtmlLangMutation loadFile ', file.locale, noload, file.name);
          });
      }
    }
    else {
      var locale = this.model.getNormalizedLocale(this.model.html.lang);
      if (locale && locale !== this.model.defaultLang) {
        this.loadFile({ locale: locale }, noload)
          .then(function () {
            //console.log('browser-storage: onHtmlLangMutation loadFile ', locale, noload);
          });
      }
    }
    return false;
  },

  /**
   * `bundle-set-fetched` event handler
   *
   * Trigger checking on auto-loading
   */
  onBundleSetFetched: function (e) {
    if (!this.file) {
      this.onHtmlLangMutationBindThis();
    }
    if (!this.autoLoadStatus.bundleSetFetched) {
      this.autoLoadStatus.bundleSetFetched = true;
      this.checkAutoLoad(this.file, this.autoLoad);
    }
  },

  /**
   * `load-xliff` event handler
   *
   * Auto-save the file if `autoSave` is enabled
   */
  onLoadXliff: function (e) {
    var locale = e.detail.locale;
    if (this.autoSave) {
      this.save({ locale: locale });
    }
    return false;
  },

  /**
   * `tap` event handler of the icon
   *
   * Hand the event to `load()` but do nothing in `load()` for now.
   */
  onTap: function (e) {
    this.load(e);
  },

  /**
   * observer of `file`
   *
   * Update label of the icon
   *
   * @param {Object} file
   */
  updateLabel: function (file) {
    if (file) {
      this.label = file.name;
      this.selected = true;
    }
    else {
      this.label = 'Browser Storage';
      this.selected = false;
    }
  },

  /**
   * observer of `file` and `autoLoad`
   *
   * Check `autoLoad` status and trigger loading of XLIFF
   *
   * @param {Object} file
   * @param {Boolean} autoLoad 
   */
  checkAutoLoad: function (file, autoLoad) {
    if (autoLoad && file && file.locale &&
        file.locale !== this.model.defaultLang && 
        this.autoLoadStatus.bundleSetFetched &&
        !this.autoLoadStatus[file.locale]) {
      this.tooltip = (this.autoSave ? 'Loaded and then Saved XLIFF for ' : 'Loaded XLIFF for ') + file.locale;
      this.autoLoadStatus[file.locale] = true;
      this.loadFile(file)
        .then(function () {
          if (!this.autoSave) {
            this.$.tooltip.show();
            setTimeout(function () {
              this.$.tooltip.hide();
              this.tooltip = '';
            }.bind(this), 3000);
          }
        }.bind(this));
    }
  },

  /**
   * load XLIFF according to the triggering event
   *
   * Events and Operations:
   * | event | origin | operation |
   * |:------|:-------|:----------|
   * | `tap` | `<live-localizer-storage-icon>` | no operation |
   * | `drag-and-drop` | `<live-localizer-storage-view>` | load the file |
   *
   * @param {Event} event the triggering event
   */
  load: function (event) {
    var file;
    switch (event.type) {
    case 'tap':
      break;
    case 'drag-and-drop':
      file = this.file;
      break;
    default:
      break;
    }
    if (file) {
      this.loadFile(file)
        .then(function () {
          this.tooltip = (this.autoSave ? 'Loaded and then Saved XLIFF for ' : 'Loaded XLIFF for ') + file.locale;
          if (!this.autoSave) {
            this.$.tooltip.show();
            setTimeout(function () {
              this.$.tooltip.hide();
              this.tooltip = '';
            }.bind(this), 3000);
          }
        }.bind(this));
    }
  },

  /**
   * load the file
   *
   * @param {Object} file the target file
   * @param {Boolean} noload true if no loading to the running app
   * @param {Boolean} nofetch true if no fetching from IndexedDB
   * @param {Promise} Promise to resolve to the file
   */
  loadFile: function (file, noload, nofetch) {
    var it = this;
    //console.log('loadFile ', file ? file.locale : null, noload, nofetch);
    if (!file) {
      file = it.model.selectedFile();
    }
    return new Promise(function (resolve, reject) {
      if (!file) {
        resolve(null);
      }
      var locale = file.locale;
      if (locale === it.model.defaultLang) {
        locale = '';
      }
      if (!locale) {
        resolve(null);
      }
      if (nofetch) {
        if (it.model.masterBundles[locale] && it.model.masterBundles[locale].bundle) {
          it.model.parseXliff(file.text, {
            bundle: noload
              ? deepcopy(it.model.masterBundles[locale])
              : it.model.masterBundles[locale]
          }, function (output, stats) {
            file.stats = stats;
            resolve(file);
          });
        }
      }
      else {
        it.storage.files.where('locale').equals(locale).toArray()
          .then(function (files) {
            var result;
            if (files && files.length > 0) {
              // found entry
              result = files[0];
              if (it.model.masterBundles[locale] && it.model.masterBundles[locale].bundle) {
                it.model.parseXliff(result.text, {
                  bundle: noload
                    ? deepcopy(it.model.masterBundles[locale])
                    : it.model.masterBundles[locale]
                }, function (output, stats) {
                  result.stats = stats;
                  if (!noload) {
                    it.storage.files.where('locale').equals(locale)
                      .delete()
                      .then(function () {
                        it.file = null;
                        it.model.updateLocale(locale);
                      })
                      .catch(function () {
                        it.file = null;
                        it.model.updateLocale(locale);
                      });
                  }
                  else {
                    it.file = result;
                  }
                  resolve(result);
                });
              }
            }
            else {
              // no entry
              it.file = null;
              resolve(null);
            }
          }, function (error) {
            reject(error);
          });
      }
    });
  },

  /**
   * update the badge label and the color according to the file's statistics
   */
  updateStats: function () {
    var badgeColorDiscarded = '';
    var stats = {};
    if (this.file && this.file.stats) {
      stats = this.file.stats.json.total;
      if (stats && stats.discarded > 0) {
        badgeColorDiscarded = 'yellow';
      }
    }

    this.badgeLabel = '' + (stats && stats.discarded > 0 ? stats.discarded : '');
    this.badgeColor = badgeColorDiscarded;
  },

  /**
   * save XLIFF to IndexedDB
   *
   * Notes:
   * - Only `file.locale` is used in the process
   *
   * @param {Object} file the target file to save to IndexedDB
   * @return {Promise}
   */
  save: function (file) {
    var it = this;
    file.text = null;
    return new Promise(function (resolve, reject) {
      it.model.setXliff(file, function (result) {
        if (result && result.text) {
          it.loadFile(result, true, true)
            .then(function (file) {
              it.storage.files.put(file)
                .then(function (f) {
                  it.file = file;
                  if (!it.tooltip) {
                    it.tooltip = 'Saved XLIFF for ' + file.locale;
                  }
                  it.$.tooltip.show();
                  setTimeout(function () {
                    it.$.tooltip.hide();
                    it.tooltip = '';
                  }, 3000);
                })
                .catch(function (e) {
                  it.file = null;
                });
            });
        }
      });
    });
  }
});
